class Bullets {
  int n;
  float[] x, y, dx, dy;
  int i;
  
  Bullets(int n) {
    this.n = n;
    x = new float[n];
    y = new float[n];
    dx = new float[n];
    dy = new float[n];
    i = 0;
  }
  
  void shoot(float X, float Y) {
    if(i < n) {
      x[i] = X; 
      y[i] = Y;
      dx[i] = random(-5, 5);
      dy[i] = sqrt(20 - dx[i]*dx[i]);
      ++i;
    }
  }
  
  void step() {
    for(int k=0; k<i; ++k) {
      x[k] += dx[k];
      y[k] -= dy[k];
      if(y[k] > height+20 || y[k] < -20 || x[k] < -20 || x[k] > width+20) {
        --i;
        x[k] = x[i];
        y[k] = y[i];
        dx[k] = dx[i];
        dy[k] = dy[i];
        --k;
      }
    }
  }
  
  void draw() {

    strokeWeight(1.5);
    noFill();
    for(int k=0; k<i; ++k) {
      stroke((k%3==0?255:128), (k%3==1?255:128), (k%3==2?255:128));
      
      line(x[k], y[k], x[k]+dx[k], y[k]-dy[k]);
  }}}

Bullets bullets;

class Ring {
  float A, B, r;
  
  Ring(float A, float B, float r) {
    this.A = A; this.B = B; this.r = r;
  }
 
  float at(float a) { return A*a + B; }
}

class Tail {
  int n;
  float[] x;
  float[] y;
  int i;
  
  Tail(int n) { 
    this.n = n;
    x = new float[n];
    y = new float[n];
    for(int i=0; i<n; ++i)
      x[i] = y[i] = 0.0;
  }
  
  void push(float X, float Y) {
    x[i] = X;
    y[i] = Y;
    if(++i == n) i = 0;
  }
  
  int index(int k) { return i+k < n ? i+k : i+k-n; }    
}  

class Bird {
  Ring ring;
  Tail tail;
  
  int n;
  Bird[] b;
    
  Bird(Ring r, int n) {
    this.ring = r;
    this.tail = new Tail(16);
    this.n = n;
    this.b = new Bird[n];
  }
  
  void go(float x, float y, float a, boolean shoot) {
    a = ring.at(a);
    float xx = x + cos(a) * ring.r;
    float yy = y + sin(a) * ring.r;
    tail.push(xx, yy); 
    
    if(shoot) bullets.shoot(xx, yy);
    
      stroke(64);
      strokeWeight(2);
      line(x, y, xx, yy);   
      noStroke();
    if(n > 0) {
      

      fill(0, 0, 0, 32);
      ellipse(xx, yy, 800, 800);
    }

    for(int i=0; i<n; i++)
      b[i].go(xx, yy, a, shoot);
    
    for(int i = 0; i < tail.x.length; ++i) {
      fill(32 + i * 8);
      int k = tail.index(i);
      ellipse(tail.x[k], tail.y[k], i+1, i+1);
    }  
    
  }
    
  void set(float x, float y, float a, boolean shoot) {    
    for(int i=0; i<n; i++)
      b[i].go(x, y, a, shoot);
      
    stroke(255, 255,0);
    strokeWeight(8);
    noFill();
    ellipse(x, y, 32, 32);
  }
} 

Bird mousebird;
float mousea = 0.0;
float mouseda = 1.0;

void setup() {
  size(600, 600);
  mousebird = new Bird(new Ring(1, 0, 100), 4);
  mousebird.b[0] = new Bird(new Ring(-1.0, radians(360-30), 150.0), 1);
  mousebird.b[1] = new Bird(new Ring(-1.0, radians(360+30), 150.0), 1);
  mousebird.b[0].b[0] = new Bird(new Ring(2.5, 0.0, 50.0), 1);
  mousebird.b[1].b[0] = new Bird(new Ring(2.5, 0.0, 50.0), 1);
  mousebird.b[0].b[0].b[0] = new Bird(new Ring(-1.0, TWO_PI, 120.0), 0);
  mousebird.b[1].b[0].b[0] = new Bird(new Ring(-1.0, TWO_PI, 120.0), 0);
  
  mousebird.b[2] = new Bird(new Ring(1.5, radians(-45), 80.0), 1);
  mousebird.b[2].b[0] = new Bird(new Ring(-1.0, PI, 80.0), 5);
  for(int i=0; i<5; ++i) 
    mousebird.b[2].b[0].b[i] = new Bird(new Ring(0.5*i, radians(90), 30 + i*10), 0);
    
  mousebird.b[3] = new Bird(new Ring(1.5, radians( 45), 80.0), 1);
  mousebird.b[3].b[0] = new Bird(new Ring(-1.0, PI, 80.0), 5);
  for(int i=0; i<5; ++i) 
    mousebird.b[3].b[0].b[i] = new Bird(new Ring(0.8*i, radians(90), 100 + i*10), 0);
  
  mousea = 0.0;
  
  bullets = new Bullets(5000);
}

void draw() {
  background(32);
  
  float a = radians(mousea);
  mousebird.set(mouseX, mouseY, a, mousePressed);
  bullets.draw();
  bullets.step();
  
  mousea += mouseda;  
}
  
   
